function algo = paramset(codename, config, seed)
% PARAMSET(config) initializes the parameters for the optimization algo
% 

if nargin == 3
	algo.seed = -1;

	% File prefix
	algo.prefix_prob = strtok(config, '.');
	algo.prefix = strcat(algo.prefix_prob, '-', codename);

	% Basic GA Parameters
	algo.pop_size = 100;
	algo.generations = 100;

	% NSGA-II crossover and mutation Parameters
	algo.prob_re = 0.9;
	algo.prob_mu = 0.2;
	algo.eta_re = 10;
	algo.eta_mu = 20;

	% Function Evaluations limit
	algo.max_fn_evals = 0;

	% Evolution Strategy (should be set in the main algo)
	algo.evolve_method = [];

	% Archive parameters
	algo.archive_add_crit = 1.e-6;

	% Surrogate train/predict method (should be set in the main algo)
	algo.train_method = [];

	% Tunable parameters for surrogates
	algo.max_traincount = 500;
	algo.train_ratio = 0.8;
	algo.train_period = 5;
	algo.dist_crit = 1.e-3;
	%algo.models = ['rsm'; 'rbf'; 'mlp'];
	algo.models = ['rbf' ];

	% Number of clusters
	algo.n_clusters = 1;
	
	% Nearest N points for choosing surrogate
	algo.n_nearest_points = 10;

	% Accuracy of the surrogate
	algo.mse_threshold = 0.1;

	algo.counter = [];

	% Define class, so new parameters can be added through user params
	algo = class(algo, 'paramset');

	% load user defined parameters from params.m and (prob)_params.m
	try 
		text = fileread('params.m');
		eval(text);
	end
	try
		param_file = strcat(algo.prefix_prob, '_params.m');
		text = fileread(param_file);
		eval(text);
	end
		
	% Set the random seed
	if (seed < 0 && algo.seed < 0) || (seed == 0)
		algo.seed = round(rand(1)*2^32);
	elseif seed > 0
		algo.seed = seed;
	end

	% initialize random sequence
	rand('state', algo.seed);
else
	algo.seed =  [];
	algo.prefix_prob = [];
	algo.prefix = [];
	algo.pop_size = [];
	algo.generations = [];
	algo.prob_re = [];
	algo.prob_mu = [];
	algo.eta_re = [];
	algo.eta_mu = [];
	algo.max_fn_evals = [];
	algo.evolve_method = [];
	algo.archive_add_crit = [];
	algo.train_method = [];
	algo.max_traincount = [];
	algo.train_ratio = [];
	algo.train_period = [];
	algo.dist_crit = [];
	algo.models = [];
	algo.n_clusters = [];
	algo.n_nearest_points = [];
	algo.mse_threshold = [];
	algo.counter = [];
	algo = class(algo, 'paramset');
end

return
