function childpop = evolve_basicga(pop, user_data)
% EVOLVE_BASICGA(pop)
%		childpop = evolve_nsga2(pop) uses SBX and mutation operators to create
% child population
childpop = population(pop);

a1 = randperm(pop.size);
a2 = randperm(pop.size);

range = get(pop.prob, 'range');
nx = get(pop.prob, 'nx');

for i = 1:4:pop.size
	parent1 = tournament(pop, a1(i), a1(i+1));
	parent2 = tournament(pop, a1(i+2), a2(i+3));
	[c1, c2] = recombination(pop, parent1, parent2, range, nx);
	childpop.x(i,:) = c1;
	childpop.x(i+1,:) = c2;
	parent1 = tournament(pop, a2(i)  , a2(i+1));
	parent2 = tournament(pop, a2(i+2), a2(i+3));
	[c1, c2] = recombination(pop, parent1, parent2, range,  nx);
	childpop.x(i+2,:) = c1;
	childpop.x(i+3,:) = c2;
end
childpop = mutation(childpop, range, nx);


%
% Binary Tournament 
%
function parent = tournament(pop, id1, id2)
if pop.rank(id1) < pop.rank(id2)
	parent = id1;
elseif pop.rank(id2) < pop.rank(id1)
	parent = id2;
elseif rand(1) <= 0.5
	parent = id1;
else
	parent = id2;
end
return


%
% Recombination 
%
function [c1, c2] = recombination(pop, id1, id2, range, nx)
c1 = zeros(nx, 1);
c2 = zeros(nx, 1);

prob_re = get(pop.algo, 'prob_re');
eta_re = get(pop.algo, 'eta_re');

if rand(1) <= prob_re
    for i = 1:nx
		if rand(1) <= 0.5
			[y1, y2] = op_SBX(pop.x(id1,i), pop.x(id2,i), ...
					range(i,1), range(i,2), eta_re);
		else
			y1 = pop.x(id1,i);
			y2 = pop.x(id2,i);
		end
		c1(i) = y1;
		c2(i) = y2;
    end
else
    c1 = pop.x(id1,:);
    c2 = pop.x(id2,:);
end
return


%
% Mutation
%
function pop = mutation(pop, range, nx)
prob_mu = get(pop.algo, 'prob_mu');
eta_mu = get(pop.algo, 'eta_mu');

for j = 1:nx
	x_min = range(j,1);
	x_max = range(j,2);
	for i = 1:pop.size
		if rand(1) <= prob_mu
			pop.x(i,j) = op_MUTATE(pop.x(i,j), x_min, x_max, eta_mu);
		end
	end
end
return
