function [surr,mse] = train(surr, x, f, g, traincount)

if nargin == 4
	traincount = size(x,1);
end

surr.nf = size(f, 2);
surr.ng = size(g, 2);
N = size(x,1);

% Identify which points are to be used as centres for RBF
if traincount < N
	cluster = clustering(x, traincount, 'kmeans');
	[id, n_id] = centroids(cluster);
else
	id = 1:N;
	n_id = [];
end

% Building models
for m = 1:surr.n_models
	model = surr.model(m,:);
	train_func = strcat(model, '_train');

	surr.(model) = feval(train_func, x(id(:),:), [f(id(:),:) g(id(:),:)]);
	try
		if sum(surr.(model).valid) ~= surr.nf+surr.ng
			surr.valid(m) = 0;
		else
			surr.valid(m) = 1;
		end
	catch
		surr.valid(m) = 1;
	end
end
surr.trained = 1;

% Surrogate validity criterion based on MSE
mse = [];
if ~isempty(n_id)
	mse = validate(surr, x(n_id(:),:), f(n_id(:),:), g(n_id(:),:));
	for m = 1:surr.n_models
		if surr.valid(m) == 1
			if max(mse(m,:)) > get(surr.algo, 'mse_threshold')
				surr.valid(m) = 0;
			end
		end
	end
end

return
