% ------------------------------------------------------------------------
% GAS0 = NSGA-II + surrogate on archive
% ------------------------------------------------------------------------
function [pop] = gas0(config, seed)
% GAS0(config, seed) Runs Single Surrogate enabled EA.
%

if nargin == 1
	seed = -1;
end

prob = problem(config);
algo = paramset(mfilename, config, seed);

algo = set(algo, 'evolve_method', 'realga');
algo = set(algo, 'train_method', 'single');

global counter
counter = struct('fn_evals', 0);

train_period = get(algo, 'train_period');

fprintf('Starting GA\n');

% Generation 1
fprintf('Generation %d\n', 1);
pop = population(prob, algo);
pop = init(pop);
pop = eval(pop);
pop = sort(pop);

if train_period > 0
	arc = archive(pop);
	arc = train(arc);
end

disp(pop);
save(pop, 1);
plot(pop);

generations = get(algo, 'generations');
max_fn_evals = get(algo, 'max_fn_evals');

% Generations 2-N
for i = 2:generations
	fprintf('Generation %d\n', i);
	
	if train_period > 0 && rem(i-1, train_period) == 0
		pop = eval(pop, [], 1);
		arc = add(arc, pop);
		train_flag = 1;
	else
		train_flag = 0;
	end

	childpop = evolve(pop);
	if train_period > 0
		childpop = eval(childpop, arc);
		arc = add(arc, childpop);
	else
		childpop = eval(childpop);
	end

	pop = pop + childpop;
	pop = sort(pop);
	pop = reduce(pop);
	
	if train_flag == 1
		arc = train(arc);
		pop = eval(pop, arc, 1);
		pop = sort(pop);
	end
 
	disp(pop);
	save(pop,i);
	plot(pop);
	if max_fn_evals > 0 && counter.fn_evals >= max_fn_evals
		break
	end
end

% Re-evaluate the final population with true evaluations
if train_period > 0
	pop = eval(pop, [], 1);
	pop = sort(pop);
	save(pop, generations+1);
end

algo = set(algo, 'counter', counter);
%save(algo);
%save(arc);

fprintf('Counters:\n');
disp(counter);

return
